@@ -1,28 +1,6 @@
 .asArgs <- function (x)
 {
-    n <- length(x)
-    if (!n)
-        return(character())
-    else if (n == 1L) {
-        xx <- x[[1L]]
-        if (is.null(xx))
-            return(character())
-        else if (is.object(xx)) {
-        }
-        else if (!length(xx))
-            return(character())
-        else if (is.numeric(xx) || is.complex(xx))
-            return(format.default(xx, trim = TRUE, digits = 17L,
-                decimal.mark = ".", drop0trailing = TRUE))
-        else if (is.raw(xx))
-            return(sprintf("0x%02x", as.integer(xx)))
-        else if (is.logical(xx) || is.character(xx)) {
-            value <- as.character(xx)
-            if (anyNA(value))
-                value[is.na(value)] <- "NA"
-            return(value)
-        }
-    }
+    if (!length(x)) return(character())
     value <- rapply(x, function(xx) {
         if (is.null(xx))
             character()
@@ -45,7 +23,7 @@
         else if (is.pairlist(xx))
             .asArgs(as.vector(xx, "list"))
         else as.character(xx)
-    }, how = "replace")
+    }, how = "list")
     value <- unlist(value, recursive = TRUE, use.names = FALSE)
     # if (anyNA(value))
     #     value[is.na(value)] <- "NA"
@@ -58,39 +36,28 @@ asArgs <- function (...)
 .asArgs(list(...))
 
 
-fileArgs <- function ()
-{
-    n <- .External2(.C_getframenumber)
-    if (n) {
-        if (n == 1L) {
-            if (.has.shFILE)
-                character()
-            else commandArgs(trailingOnly = TRUE)
-        }
-        else if (identical(sys.function(n - 1L), withArgs))
-            get("args", envir = sys.frame(n - 1L), inherits = FALSE)
-        else character()
-    }
-    else if (.has.shFILE)
-        commandArgs(trailingOnly = TRUE)
-    else character()
-}
-
-
 progArgs <- function ()
 {
     n <- .External2(.C_getframenumber)
     if (n) {
-        if (n == 1L) {
-            if (.has.shFILE)
-                character()
-            else commandArgs(trailingOnly = TRUE)
+        if (n > .toplevel.nframe()) {
+            value <- character()
+            whiches <- .External2(.C_sys.whiches, n)
+            ## remove the last element and reverse
+            if (n <- length(whiches))
+                whiches <- whiches[seq.int(to = 1L, by = -1L, length.out = n - 1L)]
+            for (which in whiches) {
+                if (.identical(sys.function(which), withArgs)) {
+                    value <- get("args", envir = sys.frame(which), inherits = FALSE)
+                    break
+                }
+            }
+            value
         }
-        else if (identical(sys.function(n - 1L), withArgs))
-            get("args", envir = sys.frame(n - 1L), inherits = FALSE)
+        ## in the site-wide startup profile file or a user profile
         else character()
     }
-    else if (.in.shell)
+    else if (.in_shell)
         commandArgs(trailingOnly = TRUE)
     else character()
 }
