@@ -22,11 +22,15 @@ extern int IS_ASCII(SEXP x);
 
 extern Rboolean R_existsVarInFrame(SEXP rho, SEXP symbol);
 
+extern SEXP ddfind(int i, SEXP rho);
+
 
 #if R_version_less_than(3, 0, 0)
 #define XLENGTH LENGTH
 #define xlength length
 #define R_xlen_t R_len_t
+#define R_XLEN_T_MAX R_LEN_T_MAX
+#define asXLength asLength
 #endif
 
 
@@ -138,6 +142,11 @@ extern SEXP R_BlankScalarString;
 #endif
 
 
+#if R_version_less_than(3, 2, 0)
+#define installChar(x) install(CHAR((x)))
+#endif
+
+
 #if R_version_less_than(3, 4, 0)
 #define R_CurrentExpression R_NilValue
 #endif
